// EEPROM handler routines for AVR
// compiler OPTIMALISATION MUST be: -o3!!!!
#include <avr/io.h>
#include <avr/interrupt.h>
#include "eeprom_hlr.h"

extern void EEPROM_MWR_WR(void);
//volatile unsigned char EEROM_WRbuffer[16];
volatile unsigned char  *EEPROMbufp;
volatile  char  EEPROMdata_length;

//volatile unsigned char EEPROM_WR_END;
static volatile char EEwrp;
volatile unsigned int EEaddr;

//use TIMER0 
//        D7  D6  D5  D4   D3     D2    D1   D0
//EECR:   -   - EEPM1 EEPM0 EERIE EEMPE EEPE EERE
//		  0   0   0   0     1     0     0    0  IT enable
//        0   0   0   0     1     1     0    0	WR_sequence1
//        0   0   0   0     1     1     1    0  WR_sequence2
//        0   0   0   0     0     0     0    1  RD

#define EEwrStart (1 << EERIE)
#define EErd (1 << EERE)
#define EE_ITdis ~(1 << EERIE)

void EEPROM_Init(void)
{
	EECR = 0x00;
	EEPROM_WR_END = 1;
}

ISR(EE_READY_vect)
{
unsigned char data;
	if(EEPROMdata_length>0)
	{
		data = *(EEPROMbufp + EEwrp);

		EEARL = (unsigned char)(EEaddr & 0x00ff);
		EEARH = (unsigned char)((EEaddr & 0x0ff00) >> 4);
		EEDR = data;
		
		//EECR = EEwrseq1; // compiler OPTIMALISATION MUST be: -o3!!!!
		//EECR = EEwrseq2;
		EEPROM_MWR_WR();
		
		EEaddr++;
		EEwrp++;
		EEPROMdata_length--;
	}
	else
	{
		EECR &= EE_ITdis;
		EEPROM_WR_END = 1;
	}
}

void WAIT_EEPROM_RDY(void)
{
char tmp;
do{
	tmp = EECR & (1<<EEPE);
  }while(tmp!=0);
}


// programmed (not IT) handling:
void EEPROM_WR(int addr, unsigned char data)
{
	WAIT_EEPROM_RDY();
	EEARL = (unsigned char)(addr & 0x00ff);
	EEARH = (unsigned char)((addr & 0x0ff00) >> 4);
	EEDR = data;
	cli();
	EECR = 1 << EEMPE;
	EECR = 1 << EEPE;
	sei();
}

unsigned char EEPROM_RD(int addr)
{
	WAIT_EEPROM_RDY();
	EEARL = (unsigned char)(addr & 0x00ff);
	EEARH = (unsigned char)((addr & 0x0ff00) >> 4);
	EECR = 1<<EERE;
	return(EEDR);
}

void EEPROM_WR_START(unsigned char  *buf, unsigned char addr, int length)
{
	EEPROMdata_length = length;
	WAIT_EEPROM_RDY();
	EEPROMbufp = buf;
	EEaddr = addr;
	EEwrp = 0;
	EEPROM_WR_END=0;
	EECR = EEwrStart;	// IT enabled
}

