// **** UART MINTA PROGRAM ****

// konyvtar header file-ok
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include "BooleConst.h"
#include "UART_hlr.h"

// konstans definiciok

//length of UART input buffer
#define serin_lng (unsigned char)16         
//length of UART output buffer
#define sero_lng  (unsigned char)64

// MACRO definiciok
//#define TRANSMIT_START TR_END=0; UCSR1B = UCSR1B | (1<<UDRIE1);
//#define TRANSMIT_STOP UCSR1B = UCSR1B & ~(1<<UDRIE1);

// globalis valtozo deklaraciok

unsigned char dumy;
//#ifdef debug
//unsigned char *memory_start =(unsigned char *) 0x2000; //explicit memoria cim megadasa

static unsigned char  SER_IN[serin_lng]; /* input buffer */
static unsigned char  SER_OU[sero_lng];  /* output buffer */

unsigned char  *seribuf; /* cyclic input buffer */
volatile unsigned char  wp;                      /* write pointer */
volatile unsigned char  rp;                      /* read pointer */
volatile unsigned char  sin_num;                 /* data number in input buffer */
unsigned char  *serobuf; /* outpput buffer */
volatile volatile unsigned char  trp;
volatile unsigned char TR_END;                  /* serial output finished */

char  serou_leng;

unsigned char  work;
unsigned int  wwork;

void WAIT_SHREND(void)
{
char tmp;
// waiting for data shift out
	do
	{
		tmp = UCSR0A;
		tmp = tmp & (1<<TXC0);
	}
	while (tmp == 0);
}


//#define UBRR_VALUE  (unsigned int)((CLK/(16*BAUD)-1) & 0x0fff)
// CLK: MHz
void USART0_Init(unsigned int baud_set_value,unsigned char dspeed)
{
	/* Set baud rate */

	UBRR0H = (unsigned char)(baud_set_value>>8);
	UBRR0L = (unsigned char)baud_set_value;
	
	if( dspeed)
	{
		UCSR0A = 1 << U2X0;
	}
	else
	{
		UCSR0A = 0x00;
	}

	UCSR0C = (1<<UCSZ01)|(1<<UCSZ00); // 8 bit adat 1 stop bit
	UCSR0B = (1<<RXEN0)|(1<<TXEN0)|(1<< RXCIE0); // tr. & rec en, rec. IT en.
	
	TR_END = 1;

	sin_num = 0;
	wp=0;
	rp=0;
	seribuf = &SER_IN[0];
	trp = 0;
	serobuf = &SER_OU[0];
}



// Transmit IT
ISR(USART_UDRE_vect) //USART data reg. empty, UDRE
{
char sdat,tmp;
	//PORTD=PORTD ^ 0x02;  LED flash 
     
    if (trp < (serou_leng))
    {
     	sdat=*(serobuf+trp);
		//sdat = SER_OU[trp];
	   	UDR0=sdat;
	   	trp+=1;
    }
    else
    {
		tmp = UCSR0B;
        tmp = tmp & ~(1<<UDRIE0);
		UCSR0B = tmp;
		TR_END=1;
        trp=0;
    }		
}

static volatile unsigned char uart_err;
//receiver IT
ISR(USART_RX_vect) 		// vteli interrupt rutin
{
char dumy;
	dumy = UCSR0A; 	   		// a statust ki kell olvasni az adat elott!
	if(dumy & (1<<FE0))
	{
		uart_err = 1<<FE0;
	}
	if(dumy & (1<<DOR0))
	{
		uart_err = 1<<DOR0;
	}							
    *(seribuf+wp) = UDR0;   // adat a bufferbe
	//SER_IN[wp] = UDR1;
    wp =(wp+1) % serin_lng;	// pointer modulo lltsa
    if (sin_num<serin_lng)	// ha nincs fellrs
    	{
       		sin_num+=1;
       	}
    else					// ha fellrtuk a legrgebbit
       	{  
 			//buf_ovr = 1;	// jelzs, tele a buffer, elvesz a legrgebbi adat!
 			sin_num = serin_lng;
 			rp = wp;		// olvas pointer az r utn mutat
 							// mivel mostmr ez a legrgebbi adat
		}
}


// bufferbol olvaso rutin
//unsigned char GET_UART(void)
unsigned char GET_UART(unsigned char * UARTvar)
{
    if (sin_num>0)
    {
		*UARTvar = *(seribuf+rp);
        rp = (rp+1) % serin_lng;
        sin_num-=1;
        return(1);
	}
    else
    {
        return(0);
    }
}

void WAIT_TREND(void)
{
unsigned char tmp;
	do
	{
		tmp=TR_END;
	}
	while(tmp == 0);
}

void TRANSMIT_START(unsigned char  *buf, char length)
{
char tmp;
	
	serobuf = buf;
	serou_leng = length;
	TR_END=0;

// clear TXC1
	tmp = UCSR0A;
	tmp = tmp | (1<<TXC0);
	UCSR0A = tmp;	

// enable data empty IT	
	tmp = UCSR0B;
	tmp = tmp | (1<<UDRIE0);
	UCSR0B = tmp;
}


void sout_byte(unsigned char bdat)
{
  	WAIT_TREND();
  	//serou_leng=1;
  	SER_OU[0]=bdat;
  	TRANSMIT_START(&SER_OU[0],1);
}

char bin_hex(char binb)
{
char hex_chr;

     binb&=0x0f;
     if (binb<=0x09) hex_chr=binb+'0';
     else            hex_chr=binb+'A'-10;
	 return(hex_chr);
}

void sout_hexbyte(unsigned char wdat)
{
  	WAIT_TREND();
  	//serou_leng=2;
  	SER_OU[0]=bin_hex((char)((wdat & 0xf0) >> 4));
  	SER_OU[1]=bin_hex((char)(wdat & 0x0f));
  	TRANSMIT_START(&SER_OU[0],2);
}

void sout_hexword(unsigned int wdat)
{
  	WAIT_TREND();
  	//serou_leng=4;
  	SER_OU[0]=bin_hex((char)((wdat & 0xf000) >> 12));
  	SER_OU[1]=bin_hex((char)((wdat & 0x0f00) >> 8));
  	SER_OU[2]=bin_hex((char)((wdat & 0x00f0) >> 4));
  	SER_OU[3]=bin_hex((char)(wdat & 0x000f));
  	TRANSMIT_START(&SER_OU[0],4);
}

void SerouString(const char *input_str)
{
char l;

	WAIT_TREND();
	strcpy(SER_OU,input_str);
	l= strlen(SER_OU);
	TRANSMIT_START(&SER_OU[0],l);
}

void CrLf(void)
{
	WAIT_TREND();
	SER_OU[0] = 0x0d;
	SER_OU[1] = 0x0a;
	//serou_leng = 2;
	TRANSMIT_START(&SER_OU[0],2);
}
