/*
 * ServoPWM.c
 *
 * Created: 3/13/2015 3:13:51 PM
 *  Author: Zoltan Benesoczky
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <stdlib.h>
#include <string.h>
#include "proj_inc.h"

//use TIMER2
//        D7     D6     D5     D4     D3    D2   D1    D0
//TCCR2A: COM2A1 COM2A0 COM2B1 COM2B0 -     -    WGM21 WGM20
//        0      0      1      0      0     0    0     1
//TCCR2B: FOC2A  FOC2B  -      -      WGM22 CS22 CS21  CS20
//        0      0      0      0      0     1    1     0
//Phase correct 8 bits PWM mode (TOP=0xff), div=256, (time resolution (period of pwm clock) 1000*256/8000000 = 0.032ms, cycle time: 2*0.032*256 = 16.384ms)
//minimum position: TH = 1.024msec when OCR2B = 16 (16*0.032)*2
#define mode2a 0x21
#define mode2b 0x06

#if Fcpu == 8000000
	#define FcpuCorr 1
#else
	#define FcpuCorr 2
#endif

//set position: 0-31
void SetServoPosition(unsigned char pos)
{
	pos = pos & 0x1f;
	SetPWM2(FcpuCorr*(16 + pos));	//Minimum position: THmin= 16*0.032*2 =1ms,  maximum position: THmax=(16+31)*0.032*2 = 3msec (cycle time = 16.032msec)
}


#define ServoPort PORTD
#define ServoPortDir DDRD
#define ServoTrig 3
#define InitServo ServoPortDir|= (1<<ServoTrig)

static volatile int servo_pos;

#ifdef swpwm
#define SetServoTrig ServoPort |= (1<<ServoTrig)
#define ClrServoTrig ServoPort &= ~(1<<ServoTrig)
//16msec
#define pwm_cnt_max (int)40
//1msec
#define pwm_min (int)2
static volatile int pwm_cnt;

void SW_PWM(void)
{
	if(pwm_cnt < pwm_cnt_max)
	{
		pwm_cnt++;
	}
	else
	{
		pwm_cnt = 0;
		SetServoTrig;
	}
	
	if(pwm_cnt == pwm_min + servo_pos)
	{
		ClrServoTrig;
	}
}
#endif

void InitCard(void)
{
	TIMER0_Init();	//0-512msec IT period
	LCD_init();
	InitServo;		// servo port direction: out
	KeyInit();
	#ifndef swpwm
	Tim2Init(mode2a, mode2b);		//set timer2 to phase sinchron PWM mode
	#endif
	sei();
}


#ifdef swpwm
#define posmax 4
#else
#define posmax 31
#endif

int main(void)
{
	#define servo_speed 200
	volatile uint8_t dir, brightness, servo_on;
	unsigned char key;
	
	InitCard();
	servo_pos = 0;
	servo_on = FALSE;
	brightness = 8;
	LCD_light(brightness);
	LCD_write_str("   Servo demo   ", line1);
	LCD_write_str_int("position: ", servo_pos, right, line2);
	dir = 1;
	SetServoPosition(servo_pos);
	
	while(1)
	{
		if(KeyPressed(&key))
		{				
			if(key == KBT1)
			{
				servo_on = ~servo_on;
				StartTime0(servo_speed);
			}
		}
		
		if(servo_on)
		{
			if(Time0Done())
			{				
				if(dir == 1)
				{
					if(servo_pos < posmax)
					{
						servo_pos = (servo_pos + 1);
						#ifndef swpwm
						SetServoPosition(servo_pos);
						#endif
					}
					else
					{
						servo_pos = (servo_pos - 1);
						#ifndef swpwm
						SetServoPosition(servo_pos);
						#endif
						dir = 0;
					}
				}
				else
				{
					if(servo_pos > 0)
					{
						servo_pos = (servo_pos - 1);
						#ifndef swpwm
						SetServoPosition(servo_pos);
						#endif
					}
					else
					{
						servo_pos = (servo_pos + 1);						
						#ifndef swpwm
						SetServoPosition(servo_pos);
						#endif
						dir = 1;
					}
				}
				LCD_write_str_int("position: ", servo_pos, right, line2);
				StartTime0(servo_speed);
			}			
		}
	}
}