//I2C functions:
#include <avr/io.h>
#include "I2C_hlr.h"

//I2C status codes:
#define START_OK 0x08
#define RPSTART_OK 0x10

#define SLA_W_ACK 0x18
#define SLA_W_NACK 0x20
#define DAT_W_ACK 0x28
#define DAT_W_NACK 0x38

#define SLA_R_ACK 0x40
#define SLA_R_NACK 0x48
#define DAT_R_ACK 0x50
#define DAT_R_NACK 0x58

unsigned char GetI2C_Stat(void)
{
	unsigned char stat;
	stat = TWSR & 0xf8;
	return(stat);
}

void TWI_Init(unsigned char br,unsigned char presc)
{
// set bit rate to 100kHz
TWBR = br; 
TWSR = presc;
}


unsigned char StartCond(void)
{
	unsigned char tmp;
	unsigned char st;

	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTA) ; // start condition
	do
	{
		tmp = TWCR & (1<<TWINT);
	}
	while(tmp==0);
	
	st = GetI2C_Stat();
	if((st == START_OK) | (st == RPSTART_OK))
	{
		return(1);
	}
	else
	{
		return(0);
	}
}

void StopCond(void)
{
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO); 
}

unsigned char TWI_DATA_WRITE(unsigned char data)
{
	unsigned char tmp;
	unsigned char st;	
	
	TWDR = data;
	TWCR = (1<<TWINT)|(1<<TWEN);
	do
	{
		tmp = TWCR & (1<<TWINT);
	}while(tmp==0);
	
	st = GetI2C_Stat();
	if((st == SLA_W_ACK) | (st == SLA_R_ACK) | (st == DAT_W_ACK))
	{
		return(1);
	}
	else
	{
		return(0);
	}
}

unsigned char TWI_DATA_READ(unsigned char* data, char ack)
{
	unsigned char tmp;
	unsigned char st;
	
	if (ack)
	{
		TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWEA);
	}
	else
	{
		TWCR = (1<<TWINT) | (1<<TWEN);
	}
	do
	{
		tmp = TWCR & (1<<TWINT);
	}while(tmp==0);
	
	st = GetI2C_Stat();
	
	if(ack)
	{
		if(st == DAT_R_ACK)
		{
			*data = TWDR;
			return(1);
		}
		else
		{
			return(0);
		}
	}
	else
	{
		if(st == DAT_R_NACK)
		{
			*data = TWDR;
			return(1);
		}
		else
		{
			return(0);
		}		
	}
}
