#include <avr/io.h>
#include <avr/interrupt.h>
#include "Fcpu.h"
#include "Timer_std_hlr.h"

//use TIMER0
//        D7     D6     D5     D4     D3    D2   D1    D0
//TCCR0A: COM0A1 COM0A0 COM0B1 COM0B0 -     -    WGM01 WGM00
//        0      0      1      0      0     0    1     1
//TCCR0B: FOC0A  FOC0B  -      -      WGM02 CS02 CS01  CS00
//        0      0      0      0      1     1    0     0
//Fast PWM mode, div=256, ORC0A = OCR0A_value (OCR0A_value*256/Fcpu = 0.512 msec, 1.95kHz)
//TIM0 counts 0...OCR0A


#if Fcpu == 16000000
	#define OCR0A_value 32
#else
	#define OCR0A_value 16
#endif

//#define OCR0A_value 32

#define tim0_modeA 0x23	//PWM on OC0B
#define tim0_modeB 0x0c

static volatile int tim0;
static volatile char ovr0;

static volatile int tim1;
static volatile char ovr1;

static volatile int tim2;
static volatile char ovr2;

static volatile int tim3;
static volatile char ovr3;

static volatile int tim4;
static volatile char ovr4;

void TIMER0_Init(void)
{
	TCCR0A = tim0_modeA;
	TCCR0B = tim0_modeB;
	OCR0A = OCR0A_value;  //f = 1.95kHz (0.512ms)
	TIMSK0 = TIMSK0 | (1 << OCIE0A); // output compare IT enabled
	
	DDR_PWM0 |= (1 << PWM0);			// PWM direction: out
	PORT_PWM0 &= ~(1 << PWM0);		// PWM = 0
}

void Time0_cnt_in_IT(void)
{
	if(tim0 > 0)
	{
		tim0--;
	}
	else
	{
		ovr0 = 1;
	}
}

void StartTime0(int t)
{
	tim0 = 2*t;
	ovr0 = 0;
}

unsigned char Time0Done(void)
{
	unsigned char timeout;
	timeout = ovr0;
	if(ovr0)
	{
		ovr0 = 0;
	}
	return(timeout);
}


void Time1_cnt_in_IT(void)
{
	if(tim1 > 0)
	{
		tim1--;
	}
	else
	{
		ovr1 = 1;
	}
}

void StartTime1(int t)
{
	tim1 = 2*t;
	ovr1 = 0;
}

unsigned char Time1Done(void)
{
	unsigned char timeout;
	timeout = ovr1;
	if(ovr1)
	{
		ovr1 = 0;
	}
	return(timeout);
}


void Time2_cnt_in_IT(void)
{
	if(tim2 > 0)
	{
		tim2--;
	}
	else
	{
		ovr2 = 1;
	}
}

void StartTime2(int t)
{
	tim2 = 2*t;
	ovr2 = 0;
}

unsigned char Time2Done(void)
{
	unsigned char timeout;
	timeout = ovr2;
	if(ovr2)
	{
		ovr2 = 0;
	}
	return(timeout);
}

//Time3
void Time3_cnt_in_IT(void)
{
	if(tim3 > 0)
	{
		tim3--;
	}
	else
	{
		ovr3 = 1;
	}
}

void StartTime3(int t)
{
	tim3 = 2*t;
	ovr3 = 0;
}

unsigned char Time3Done(void)
{
	unsigned char timeout;
	timeout = ovr3;
	if(ovr3)
	{
		ovr3 = 0;
	}
	return(timeout);
}

//Time4
void Time4_cnt_in_IT(void)
{
	if(tim4 > 0)
	{
		tim4--;
	}
	else
	{
		ovr4 = 1;
	}
}

void StartTime4(int t)
{
	tim4 = 2*t;
	ovr4 = 0;
}

unsigned char Time4Done(void)
{
	unsigned char timeout;
	timeout = ovr4;
	if(ovr4)
	{
		ovr4 = 0;
	}
	return(timeout);
}



// Timer0 compareA IT, 1.95kHz (0.512ms)
ISR(TIMER0_COMPA_vect)
{
	Time0_cnt_in_IT();
	Time1_cnt_in_IT();
	Time2_cnt_in_IT();
	Time3_cnt_in_IT();
	Time4_cnt_in_IT();		
	Timer_std_ItExtension();
}



/*
void DelayTim0(int t)
{
	tim0 = t;
	ovr0 = 0x00;
	do
	{
	}while(!ovr0);
}
*/

void SetPWM0(unsigned char pwm)
{
	if(pwm == 0)
	{
		TCCR0A &= ~(1 << COM0B1);
	}
	else
	{
		TCCR0A |= (1 << COM0B1);
		OCR0B = pwm & 0x0f;
	}
}

void Tim2Init(unsigned int mode2a_, unsigned int mode2b_)
{
	TCCR2A = mode2a_;
	TCCR2B = mode2b_;
}

void SetPWM2(unsigned char pwm2)
{
	OCR2B = pwm2;
}