//(C) Zoltan Benesoczky
// Sample program for Ultrasonic distance measurement with AVR and HC-SR4 UH sensor
//-----------------------------------------

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <string.h>
#include "proj_inc.h"

// calculating I2C bit rate register TWBR for fscl = 400kHz
#define Fi2c (long)400000
#define Fcpu (long)8000000
#define i2cbr (unsigned char)(Fcpu/(2*Fi2c)-8)
#define BAUD (long)9600	//Baud
#define UBRR_VALUE  (unsigned int)((Fcpu/(16*BAUD)-1) & 0x0fff)

float RefValue;
#define DistLimitAddr 0
unsigned int DistLimit;

unsigned char ebuf[2];
void EEPROM_WR_Int(int data,int addr)
{
	ebuf[0] = (unsigned char)(data & 0x00ff);
	ebuf[1] = (unsigned char)(data >> 8);
	EEPROM_WR_START(&ebuf[0],addr,2);
}

int EEPROM_RD_Int(int addr)
{
	int result;
	result = EEPROM_RD(addr);
	result |= (EEPROM_RD(addr+1) << 8);
	return(result);
}

void SysIni(void)
{
	unsigned char RefSelect;
	TIMER0_Init();
	TWI_Init(i2cbr, PrescVal_1);
	EMC1001_Ini(Hz4);		// set refresh rate of temperature sensor
	RefSelect = Ref_1V1;
	RefValue = 1.1;
	/*
	RefSelect = Ref_AVCC;
	RefValue = 3.3;
	*/
	ADC_Init(RefSelect, ADC_Temp, Trg_FreeRun, fdiv_128, 1);
	USART0_Init(UBRR_VALUE);
	LCD_init();
	KeyInit();
	KeyScanEnabled = TRUE;
	EEPROM_Init();
}

#define FlashTime 250
#define BrNormal 5
#define BrMax 15
void LCD_Flash(unsigned char reset)
{
	static unsigned char toggle;
	if(reset)
	{
		LCD_light(BrNormal);
		StartTime4(FlashTime);
		toggle = 0;
	}
	else
	{
		if(Time4Done())
		{			
			//toggle = ~toggle;
			if(toggle)
			{
				LCD_light(BrMax);
				toggle = 0;
			}
			else
			{
				LCD_light(BrNormal);
				toggle = 1;
			}
			StartTime4(FlashTime);
		}		
	}
}

int GetInteger(void)
{	unsigned char data;
	char intstr[8];
	int i = 0;
	do{
		do{}while(!GET_UART(&data));
		if((data == CR) | (i > 5))
		{
			break;
		}
		else
		{
			intstr[i] = data;
			i++;
		}
	}while(1);
	intstr[i] = 0;
	return (atoi(intstr));
}


// Remote commands:
#define SetRemoteMode 'R'
#define ClrRemoteMode 'C'
#define GetTemp 'T'
#define GetDist 'D'
#define GetVoltage 'V'
#define GetLimit 'L'
#define SetLimit 'S'

static char AnswStr[16];
//
#define TempRefresh 500
#define DisplayRefresh 100
//
#define KBT_RET KBT1
#define KBT_INC KBT2
#define KBT_DEC KBT3

#define TempMode 0
#define DistMode 1
#define DistLimMode 2
#define VoltageMode 3

//LCD line1/2:
//" Temperature"
//"          25.6 C"

//"   Distance "
//"250 cm    25.6 C"

//" Dist. limit"
//"250 cm    25.6 C"

//"    Voltage "
//"3.3 V     25.6 C"

#define rem_pos 14

#define Temp_pos 11
#define Temp_with 4
#define Temp_prec 1
#define C_pos 15

#define Dist_pos 0
#define cm_pos 3

#define Volt_pos 0
#define Volt_with 4
#define Volt_prec 1
#define V_pos 4

#define DistLimMax 300
volatile unsigned char RemoteMode;
void ModeFSM(unsigned char reset)
{			
	static unsigned char ModeState;
	static unsigned char LimitSetEnabled;
	static unsigned char alert;
	static double td;	
	static DistMeasResult DistResult;
	static double AD_value;
	static unsigned char send_Temp,send_Dist,send_Lim,send_Voltage;	
	unsigned char button;
	//static unsigned char RemoteMode;
	
	unsigned char UART_data;
	unsigned char tmp;


	
	if(reset)
	{
		ModeState = TempMode;
		LCD_write_str(" Temperature",line1);
		LCD_write_str_pos("C",line2, C_pos);	
		StartTime0(TempRefresh);
		LimitSetEnabled = FALSE;
		alert = FALSE;
		LCD_Flash(TRUE);
		RemoteMode = FALSE;
		send_Temp = FALSE;
		send_Dist = FALSE;
		send_Lim  = FALSE;
		send_Voltage  = FALSE;
	}
	else
	{		
		//Decode and enable to execute remote commands	
		if (GET_UART(&UART_data))
		{
			if(RemoteMode)
			{
				switch(UART_data)
				{				
					case GetTemp:
						LCD_write_str_pos(" Temperature",line1,0);
						send_Temp = TRUE;						
						ModeState = TempMode;
						break;
					case GetDist:
						LCD_write_str_pos("   Distance ",line1,0);
						StartTime3(DisplayRefresh);
						send_Dist = TRUE;
						ModeState = DistMode;						
						break;
					case GetLimit:
						LCD_write_str_pos(" Dist. limit",line1,0);
						send_Lim = TRUE;
						ModeState = DistLimMode;
						break;
					case SetLimit:
						tmp = GetInteger();
						LCD_write_str_pos(" Dist. limit",line1,0);
						if((tmp > 0)&&(tmp<=DistLimMax))
						{
							DistLimit = tmp;
							EEPROM_WR_Int(DistLimit,DistLimitAddr);
						}
						send_Lim = TRUE;
						ModeState = DistLimMode;
						break;						
					case GetVoltage:
						LCD_write_str_pos("    Voltage ",line1,0);
						send_Voltage = TRUE;
						ModeState = VoltageMode;
						break;
					case ClrRemoteMode:
						RemoteMode = FALSE;
						KeyScanEnabled = TRUE;
						LCD_write_str_pos("   ",line1,rem_pos);
						strcpy(AnswStr,"End of REMOTE mode");
						SerouString(AnswStr);
						CrLf();												
						break;
					case SetRemoteMode:
						strcpy(AnswStr,"Already in REMOTE mode");
						SerouString(AnswStr);
						CrLf();
						break;						
					default:
						strcpy(AnswStr,"Not defined command!");
						SerouString(AnswStr);
						CrLf();
						break;
					}				
			}
			else
			{
				switch(UART_data)
				{
				case SetRemoteMode:
					KeyScanEnabled = FALSE;
					RemoteMode = TRUE;
					LCD_write_str_pos("REM",line1,rem_pos);
					strcpy(AnswStr,"REMOTE mode");
					SerouString(AnswStr);
					CrLf();					
					break;
				case ClrRemoteMode:
					strcpy(AnswStr,"Already not in REMOTE mode");
					SerouString(AnswStr);
					CrLf();
					break;				
				default:
					strcpy(AnswStr,"First set REMOTE mode!");
					SerouString(AnswStr);
					CrLf();					
					break;
				}	
			}
		}
				
		//Read and display temperature
		if(Time0Done())
		{
			if(Temperature(&td))
			{
				LCD_write_double_pos(td,Temp_with,Temp_prec,line2,Temp_pos);
				StartTime0(TempRefresh);
			}
		}	
			
		switch(ModeState)
		{
		case TempMode:
				if(KeyPressed(&button))
				{	
					switch(button)
					{											
					case KBT_INC:
						ModeState = DistMode;
						LCD_write_str_pos("   Distance ",line1,0);
						LCD_write_str_pos("cm  ",line2, cm_pos);
											
						break;
					case KBT_DEC:
						ModeState = VoltageMode;
						LCD_write_str_pos("    Voltage ",line1,0);
						LCD_write_str_pos("V ",line2, V_pos);
						break;
					}					
				StartTime3(DisplayRefresh);		
				}
				
				//execute remote command
				if(send_Temp)
				{
					dtostrf(td,Temp_with,Temp_prec,AnswStr);
					SerouString(AnswStr);
					CrLf();
					send_Temp = FALSE;
				}
				break;
		case DistMode:
				//DistResult = UH_DistanceMeas(FALSE);
				if(Time3Done())
				{		
					DistResult = UH_DistanceMeas(FALSE);			
					if(DistResult.valid)
					{
						LCD_write_int_pos(DistResult.value,line2,Dist_pos);
						LCD_write_str_pos("cm  ",line2, cm_pos);
						
						//execute remote command
						if(send_Dist)
						{
							itoa(DistResult.value,AnswStr,10);
							SerouString(AnswStr);
							CrLf();
							send_Dist = FALSE;
						}
						
						//compare distance limit and enable/disable alert
						if(DistResult.value > DistLimit)
						{
							alert = TRUE;
						}
						else
						{
							alert = FALSE;
						}
					}
					else
					{
						LCD_write_str_pos("No echo",line2, Dist_pos);
						
						//execute remote command
						if(send_Dist)
						{
							strcpy(AnswStr,"No echo");
							SerouString(AnswStr);
							CrLf();
							send_Dist = FALSE;
						}
						
						alert = FALSE;
					}	
					StartTime3(DisplayRefresh);					
				}
				
				if(alert)
				{
					LCD_Flash(FALSE);
				}
				else
				{
					LCD_light(BrNormal);	//reset brightness
				}
				
				if(KeyPressed(&button))
				{		
					switch(button)
					{
					case KBT_INC:
						ModeState = DistLimMode;
						LCD_write_str_pos(" Dist. limit",line1,0);
						LCD_write_str_pos("cm  ",line2, cm_pos);
						break;
					case KBT_DEC:
						ModeState = TempMode;
						LCD_write_str_pos(" Temperature",line1,0);
						LCD_write_str_pos("       ",line2, Dist_pos);
						break;
					}
					StartTime3(DisplayRefresh);
				}
				break;
		case DistLimMode:
				if(Time3Done())
				{
					LCD_write_int_pos(DistLimit,line2,Dist_pos);
					LCD_write_str_pos("cm  ",line2, cm_pos);
					

					StartTime3(DisplayRefresh);					
				}
				
				//execute remote command
				if(send_Lim)
				{
					itoa(DistLimit,AnswStr,10);
					SerouString(AnswStr);
					CrLf();
					send_Lim = FALSE;
				}									
				
				switch(LimitSetEnabled)
				{
				case FALSE:
					if(KeyPressed(&button))
					{
						switch(button)
						{
							case KBT_INC:
								ModeState = VoltageMode;
								LCD_write_str_pos("    Voltage ",line1,0);
								LCD_write_str_pos("V ",line2, V_pos);
								break;
							case KBT_DEC:
								ModeState = DistMode;
								LCD_write_str_pos("   Distance ",line1,0);
								LCD_write_str_pos("cm",line2, cm_pos);
								break;
							case KBT_RET:
								LCD_write_str_pos("Dist.lim set",line1,0);
								LimitSetEnabled = TRUE;
								break;
						}
						StartTime3(DisplayRefresh);
					}					
					break;
				case TRUE:
					if(KeyPressed(&button))
					{
						switch(button)
						{
						case KBT_INC:
							if(DistLimit < DistLimMax)
							{
								DistLimit++;								
							}
							break;
						case KBT_DEC:
							if(DistLimit > 0)
							{
								DistLimit--;								
							}
							break;
						case KBT_RET:
							LimitSetEnabled = FALSE;
							LCD_write_str_pos(" Dist. limit",line1,0);
							EEPROM_WR_Int(DistLimit,DistLimitAddr);						
							break;						
						}
					}
					break;										
				}
				break;
		case VoltageMode:				
				if( Get_ADC_Voltage(&AD_value,RefValue))
				{
					if(Time3Done())
					{
						LCD_write_double_pos(AD_value,Volt_with,Volt_prec,line2,Volt_pos);
						StartTime3(DisplayRefresh);
					}
					
					//execute remote command
					if(send_Voltage)
					{
						dtostrf(AD_value,Volt_with,Volt_prec,AnswStr);
						SerouString(AnswStr);
						CrLf();
						send_Voltage = FALSE;
					}					
				}				
								
				if(KeyPressed(&button))
				{		
					switch(button)
					{
					case KBT_INC:
						ModeState = TempMode;
						LCD_write_str_pos(" Temperature",line1,0);
						LCD_write_str_pos("     ",line2, Volt_pos);
						break;
					case KBT_DEC:
						ModeState = DistLimMode;
						LCD_write_str_pos(" Dist. limit",line1,0);
						LCD_write_str_pos("cm  ",line2, cm_pos);
						break;
					}					
					StartTime3(DisplayRefresh);
				}
				break;
		}
	}
}



int main(void)
{		
	SysIni();
	LCD_light(BrNormal);
	sei();
	
	UH_DistanceMeas_Init();
	DistLimit = EEPROM_RD_Int(DistLimitAddr);
	if((DistLimit > DistLimMax) | (DistLimit<0))
	{
		DistLimit = DistLimMax;
		EEPROM_WR_Int(DistLimit,DistLimitAddr);
		DistLimit = EEPROM_RD_Int(DistLimitAddr);
	}
		
	ModeFSM(TRUE);
	do{
		ModeFSM(FALSE);
	}while(1);
}
